@@ -1,14 +1,15 @@
 #!/usr/bin/perl
-#$Id: stat.pl 805 2011-06-27 19:58:07Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/stat/stat.pl $
+#$Id: stat.pl 908 2011-10-18 15:50:01Z pro $ $URL: svn://svn.setun.net/dcppp/trunk/examples/stat/stat.pl $
 package statpl;
 use strict;
 no warnings qw(uninitialized);
 our ( %config, %static, $param, $db, );
 use Data::Dumper;
 $Data::Dumper::Sortkeys = $Data::Dumper::Useqq = $Data::Dumper::Indent = 1;
+#use lib::abs qw(../../lib ./ pslib);
 use lib::abs qw(../../lib ./);
 use Net::DirectConnect::pslib::psmisc;    # qw(:config :log printlog);
-psmisc->import qw(:log);
+psmisc->import(qw(:log));
 #*statpl::config = *main::config;
 #our $root_path;
 #use lib $root_path. '../../lib';
@@ -72,13 +73,13 @@ for my $arg (@ARGV) {
         local $config{'queries'}{$query}{'WHERE'}[5] =
           $config{'queries'}{$query}{'FROM'} . ".time >= " . int( time - $config{'periods'}{$time} )
           if $time;
-        my $res  = statlib::make_query( { %{ $config{'queries'}{$query} }, }, $query );
-#print $query, Dumper $res;
-        my $n    = 0;
+        my $res = statlib::make_query( { %{ $config{'queries'}{$query} }, }, $query );
+        #print $query, Dumper $res;
+        my $n = 0;
         my $date = psmisc::human( 'date', $nowtime ) . ( $tim ne 'h' ? '' : '-' . sprintf '%02d', ( localtime $nowtime )[2] );
         for my $row (@$res) {
           ++$n;
-          delete $row->{$_} for grep {!defined $row->{$_}} keys %$row;
+          delete $row->{$_} for grep { !defined $row->{$_} } keys %$row;
           my $dmp = Data::Dumper->new( [$row] )->Indent(0)->Pair('=>')->Terse(1)->Purity(1)->Dump();
           #warn "SLOWi:[$config{'use_slow'}][$dmp]";
           $db->insert_hash( 'slow', { 'name' => $query, 'n' => $n, 'result' => $dmp, 'period' => $time, 'time' => $nowtime } )
@@ -99,13 +100,13 @@ for my $arg (@ARGV) {
           #}
         }
         #exit;
-        $db->do( "DELETE FROM ${tq}slow${tq} WHERE name="
+        $db->do(
+              "DELETE FROM ${tq}slow${tq} WHERE name="
             . $db->quote($query)
             . " AND period="
             . $db->quote($time)
-            #. (!$config{'sql'}{'table'}{}" AND n>$n AND ${rq}date${rq}=$vq$date$vq") 
-            )
-          if $config{'use_slow'};
+            #. (!$config{'sql'}{'table'}{}" AND n>$n AND ${rq}date${rq}=$vq$date$vq")
+        ) if $config{'use_slow'};
         #$db->flush_insert('slow');
         $db->flush_insert();
         #sleep 3;
@@ -126,6 +127,8 @@ for my $arg (@ARGV) {
         $db->do( "DELETE FROM $tq$table$tq WHERE $col < " . int( time - $purge ) );
     }
     $db->optimize() unless $config{'no_auto_optimize'};
+  } elsif ( $arg eq 'optimize' ) {
+    $db->optimize();
   } elsif ( $arg eq 'install' ) {
     $ARGV[$n] = undef;
     local $db->{error_sleep} = 0;
@@ -137,7 +140,7 @@ for my $arg (@ARGV) {
     local $db->{'auto_install'} = 0;
     local $db->{'error_sleep'}  = 0;
     #my ( $tq, $rq, $vq ) = $db->quotes();
-
+    $db->upgrade();
 =old 
     $db->do( "DROP TABLE ${_}d")    for qw(queries_top_string_ queries_top_tth_ results_top_);
     $db->do("ALTER TABLE queries_top_string_daily RENAME TO queries_top_string_d");
@@ -220,7 +223,8 @@ for ( grep { length $_ } @ARGV ? @hosts : psmisc::array( $config{dc}{host} ) ) {
     my $hub = $_;
     ++$work{'hubs'}{$hub};
     my $dc = Net::DirectConnect->new(
-      modules     => { 'filelist' => 1 },
+      #modules     => { 'filelist' => 1 },
+      db => $db, #for filelist
       'Nick'      => 'dcstat',
       'sharesize' => 40_000_000_000 + int( rand 10_000_000_000 ),
       #'log'		=>	sub {},	# no logging
@@ -267,6 +271,7 @@ for ( grep { length $_ } @ARGV ? @hosts : psmisc::array( $config{dc}{host} ) ) {
           psmisc::schedule(
             $config{'queue_recalc_every'},
             our $queuerecalc_ ||= sub {
+              my $dc   = shift;
               my $time = int time;
               $work{'toask'} = [ (
                   sort { $work{'ask'}{$b} <=> $work{'ask'}{$a} }
@@ -287,21 +292,24 @@ for ( grep { length $_ } @ARGV ? @hosts : psmisc::array( $config{dc}{host} ) ) {
                 " first hits=", $work{'ask'}{ $work{'toask'}[0] },
                 ' asks=', scalar keys %{ $work{'ask'} }
               );
-            }
+            },
+            $dc
           );
           psmisc::schedule(
             [ 3600, 3600 ],
             our $hashes_cleaner_ ||= sub {
+              my $dc = shift;
               my $min = scalar keys %{ $work{'hubs'} || {} };
               $dc->log( 'info', "queue clear min[$min] now", scalar %{ $work{'ask'} || {} } );
               delete $work{'ask'}{$_} for grep { $work{'ask'}{$_} < $min } keys %{ $work{'ask'} || {} };
               $dc->log( 'info', "queue clear ok now", scalar %{ $work{'ask'} || {} } );
-            }
+            },
+            $dc
           );
           psmisc::schedule(
             $dc->{'search_every'},
             our $queueask_ ||= sub {
-              my ($dc) = @_;
+              my $dc = shift;
               my $q;
               while ( $q = shift @{ $work{'toask'} } or return ) {
                 my $r;
@@ -504,15 +512,20 @@ while ( my @dca = grep { $_ and $_->active() } @dc ) {
     ,
     @dc
   );
-  psmisc::schedule( [ 300, 60 * 19 ], our $hubrunhour_ ||= sub {
-     psmisc::printlog( 'err', 'cant lock h'),
-     return if !psmisc::lock('calch', old=>86400);
-     psmisc::startme('calch'); } ),
-    psmisc::schedule( [ 600, 60 * 60 * 6 ], our $hubrunrare_ ||= sub {
-     psmisc::printlog( 'err', 'cant lock r'),
-     return if !psmisc::lock('calcr', old=>86400);
- psmisc::startme('calcr'); } )
-    if $config{'use_slow'};
+  psmisc::schedule(
+    [ 300, 60 * 19 ],
+    our $hubrunhour_ ||= sub {
+      psmisc::printlog( 'err', 'cant lock h' ), return if !psmisc::lock( 'calch', old => 86400 );
+      psmisc::startme('calch');
+    }
+    ),
+    psmisc::schedule(
+    [ 600, 60 * 60 * 6 ],
+    our $hubrunrare_ ||= sub {
+      psmisc::printlog( 'err', 'cant lock r' ), return if !psmisc::lock( 'calcr', old => 86400 );
+      psmisc::startme('calcr');
+    }
+    ) if $config{'use_slow'};
 #psmisc::schedule( [ 60 * 3, 60 * 60 * 24 ], our $hubrunoptimize_ ||= sub { psmisc::startme('calcr'); } )    if $config{'auto_optimize'};
   psmisc::schedule( [ 900, 86400 ], $config{'purge'} / 10, our $hubrunpurge_ ||= sub { psmisc::startme('purge'); } );
 
